// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// under GPL or new BSD license

//Silly demo of the tiny3d engine :-)
//
//Shows  a IMD bug and a MD2 bug

#include <kos.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include "tiny3d.hpp"

#include "glwrap.h"

#define FPS 60


KOS_INIT_FLAGS(INIT_DEFAULT);
#ifndef NOROMDISK
extern uint8 romdisk[];
KOS_INIT_ROMDISK(romdisk);
#endif


// get the context
Context &context = Context::Get();


t3DModel g_3DModel;	// This holds the 3DS Model info that we load in

CLoad3DS g_Load3ds;	// This is 3DS class.  This should go in a good model class.

CFrustum *g_Frustum;

//create the 3DS entity
ENTITY_3DS *ent_box = new ENTITY_3DS();

//create the IMD entity
ENTITY_IMD *ent_bug = new ENTITY_IMD();
ENTITY_IMD *ent_f = new ENTITY_IMD();
//create the MD2 entity
ENTITY_MD2 *ent = new ENTITY_MD2();



void set_projection (void)
{
    float	zNear = 0.1f;
    float	zFar = 10000;
    float	aspectratio = 640.0f / 480.0f;
    float	field_of_view = 45; // degrees


    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    gluPerspective (field_of_view, aspectratio, zNear, zFar);
}

void culling(const bool state){
    if(state) glEnable(GL_CULL_FACE);
    else
        glDisable(GL_CULL_FACE);
};


int main(int argc, char **argv) {

    uint8	c;
    maple_device_t * caddr;
    cont_state_t * cont;
    pvr_stats_t stats;


    pvr_init_params_t params = {{PVR_BINSIZE_16, PVR_BINSIZE_0, PVR_BINSIZE_16, PVR_BINSIZE_0, PVR_BINSIZE_0},1024*1024};



    float	z = 0;
    float y=0;

    float tx=0, ty=0, tz=0;
    float rx=0.0f, ry=0.0f, rz=0.0f;

    int	a_pressed = false;

    GLfloat camera[3]={0,0,0};

    GLfloat look[3]={0,0,-1};

    GLfloat up[3]={0,1,0};

    g_Frustum=context.GetFrustrum();


    pvr_init(&params);

    //init KGL and set the default viewport stuff
    glKosInit();

    //init texture.c
    texture_init();

    set_projection();


    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity();

    glEnable(GL_TEXTURE_2D);

    glShadeModel(GL_SMOOTH);

    glDisable(GL_KOS_AUTO_UV);

    // Enable back face culling
    culling(true);

    /* Expect CW verts */
    glFrontFace(GL_CCW);

    //grab controller 1
    c = maple_first_controller();


    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_FILTER, GL_FILTER_BILINEAR);

    //loading some 3ds files the 2nD "/rd" defines the texture directory

//     g_Load3ds.Import3DS(&g_3DModel,"/rd/box.3ds","/rd/",1.0f/20.f);

//     ent_box->SetModel(&g_3DModel,"box.3ds");
//     ent_box->SetPosition(0,0,-2.5);

    //     //MD2 loading
    ent->Load("/rd/bug.md2","/rd/bug.jpg",(7.0f/20.0f));

    ent->SetPosition(0,0,-3);
    ent->Position-=ent->BoundingSphere.GetCenter();
    ent->SetFPS(10/2);
    ent->SetAnimStart(0);
    ent->SetAnimEnd(30);


    ent_bug->Load("bug.imd","/rd/",(7.0f/20.0f));
    ent_bug->SetPosition(-0.6,0,-3);
    ent_bug->SetAnimStart(0);
    ent_bug->SetAnimEnd(14);

    //The Original model has 0-199 Keyframes at 60FPS
    // I reduced the keyframes to 15FPS which gave us 0-14 Keyframes
    // Now I set FPS to (15 /2) so I have the
    // intended original speed but 2x times slower.
    ent_bug->SetFPS(15/2);

    //engine loop starts here
    while(1) {



        z=y=0;

        /* Check controller/key status */
        if ((caddr = maple_enum_type(0, MAPLE_FUNC_CONTROLLER)) == NULL)
            break;

        cont = (cont_state_t *)maple_dev_status(caddr);
        if ((cont->buttons & CONT_START))
        {
            //frees the memory .just a test if DC hangs
            if(ent_box!=NULL)
                ent_box->FreeTextures();

            delete ent;
            delete ent_box;
            delete ent_bug;
            delete ent_f;
            break;
        }
        if ((cont->buttons & CONT_DPAD_LEFT))
        { y = 0.01f;

            tx += fsin(rx*F_PI/180.0f);
        }
        if ((cont->buttons & CONT_DPAD_RIGHT))
        {  y = -0.01f;

            tx -= fsin(rx*F_PI/180.0f);
        }
        if ((cont->buttons & CONT_DPAD_DOWN))
        {
            z = -0.01f;
            tz -= fcos(ry*F_PI/180.0f);

        }
        if ((cont->buttons & CONT_DPAD_UP))
        {  z= 0.01f;
            tz += fcos(ry*F_PI/180.0f);
        }
        if ((cont->buttons & CONT_A) && !a_pressed) {
            a_pressed = true;

        }
        if (!(cont->buttons & CONT_A))
            a_pressed = false;

        if(a_pressed) {
            static int count=1;
            char  buffer[256];
            sprintf(buffer,"/pc/home/shoots/shot%..ppm", count++);
            vid_screen_shot(buffer);


        }

        rx+=((float)cont->joyy)/100.0f;
        ry+=((float)cont->joyx)/100.0f;


        //tell KGL we're rendering a new frame
        glKosBeginFrame();


        //   set the camera position
        glwLookAt(camera, look,up);

        kglColor4f(1,1,1,1);

        g_Frustum->CalculateFrustum();


#if 1
        static int rotater=0;


        ent->Delta_y=ry;
        ent->Delta_x=rx;

        ent->Movement_y=y;
        ent->Movement_z=z;


        ent_box->Delta_y=ry;
        ent_box->Delta_x=rx;



        ent_bug->Delta_y=ry;
        ent_bug->Delta_x=rx;

        ent_bug->Movement_y=y;
        ent_bug->Movement_z=z;



        glPushMatrix();

        ent->Update();
        ent->Draw();
        ent->NextAnim();
        glPopMatrix();

//         glPushMatrix();
//         ent_box->Update();
//         ent_box->Draw();
//         glPopMatrix();

        glPushMatrix();
        ent_bug->Update();
        ent_bug->Draw();
        ent_bug->NextAnim();
        glPopMatrix();


#endif



        //  //tell KOS our opaque frame is done
        //         glKosFinishList();
        // 		glEnable(GL_BLEND);
        // put your blending stuff here->

        // <-
        //tell KOS our frame is done
        glKosFinishFrame();
        glDisable(GL_BLEND);

#if 0
        pvr_get_stats(&stats);
        if((stats.frame_count%1200)==0){


            printf("Frame Count: %d, last_time: %f, frame rate: %f fps\n",
                   stats.frame_count, stats.frame_last_time, stats.frame_rate);

            printf("Camera position: %f, %f, %f\n",
                   camera[0],   camera[1],   camera[2]);

        }
        //end of engine loop
#endif
    }
    //end of program
    return 0;
}

