// MATRIX Class
// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// under GPL or new BSD license
// v 0 (21.10.02): initial release
#include <dc/fmath.h>

#include "matrix.h"


MATRIX::MATRIX()
{
    LoadIdentity();
}

MATRIX::~MATRIX()
{
}

void	MATRIX::Print()
{
    for (int i = 0; i < 4; i ++)
    {
        printf("[ ");
        for (int j = 0; j < 4; j ++)
            printf("\t%3.3f ", _dcm[i][j]);
        printf("]\n");
    }
}

//FIXME all _m[0] to _11 etc pp

GLvoid MATRIX::LoadIdentity()
{
    memset(_m, 0, sizeof(float) * 16);
    _11 = _22 = _33 = _44 = 1;
}

GLvoid MATRIX::CopyMatrix(GLfloat m[16])
{
    _m[0 ] = m[0 ];
    _m[1 ] = m[1 ];
    _m[2 ] = m[2 ];
    _m[3 ] = m[3 ];
    _m[4 ] = m[4 ];
    _m[5 ] = m[5 ];
    _m[6 ] = m[6 ];
    _m[7 ] = m[7 ];
    _m[8 ] = m[8 ];
    _m[9 ] = m[9 ];
    _m[10] = m[10];
    _m[11] = m[11];
    _m[12] = m[12];
    _m[13] = m[13];
    _m[14] = m[14];
    _m[15] = m[15];
}


GLvoid MATRIX::MultMatrix(GLfloat m[])
{
    MATRIX temp;

    assert(0); //FIXME: this destroys the internal SH4 matrix!
    temp.CopyMatrix(m);

    Load();
    temp.Apply();
    Store();
#if 0
    _m[0] = temp._m[0 ]*m[0 ]
            + temp._m[4 ]*m[1 ]
            + temp._m[8 ]*m[2 ]
            + temp._m[12]*m[3 ];

    _m[1] = temp._m[1 ]*m[0 ]
            + temp._m[5 ]*m[1 ]
            + temp._m[9 ]*m[2 ]
            + temp._m[13]*m[3 ];

    _m[2] = temp._m[2 ]*m[0 ]
            + temp._m[6 ]*m[1 ]
            + temp._m[10]*m[2 ]
            + temp._m[14]*m[3 ];

    _m[3] = temp._m[3 ]*m[0 ]
            + temp._m[7 ]*m[1 ]
            + temp._m[11]*m[2 ]
            + temp._m[15]*m[3 ];

    _m[4] = temp._m[0 ]*m[4 ]
            + temp._m[4 ]*m[5 ]
            + temp._m[8 ]*m[6 ]
            + temp._m[12]*m[7 ];

    _m[5] = temp._m[1 ]*m[4 ]
            + temp._m[5 ]*m[5 ]
            + temp._m[9 ]*m[6 ]
            + temp._m[13]*m[7 ];

    _m[6] = temp._m[2 ]*m[4 ]
            + temp._m[6 ]*m[5 ]
            + temp._m[10]*m[6 ]
            + temp._m[14]*m[7 ];

    _m[7] = temp._m[3 ]*m[4 ]
            + temp._m[7 ]*m[5 ]
            + temp._m[11]*m[6 ]
            + temp._m[15]*m[7 ];

    _m[8] = temp._m[0 ]*m[8 ]
            + temp._m[4 ]*m[9 ]
            + temp._m[8 ]*m[10]
            + temp._m[12]*m[11];

    _m[9] = temp._m[1 ]*m[8 ]
            + temp._m[5 ]*m[9 ]
            + temp._m[9 ]*m[10]
            + temp._m[13]*m[11];

    _m[10]= temp._m[2 ]*m[8 ]
            + temp._m[6 ]*m[9 ]
            + temp._m[10]*m[10]
            + temp._m[14]*m[11];

    _m[11]= temp._m[3 ]*m[8 ]
            + temp._m[7 ]*m[9 ]
            + temp._m[11]*m[10]
            + temp._m[15]*m[11];
    _m[12]= temp._m[0 ]*m[12]
            + temp._m[4 ]*m[13]
            + temp._m[8 ]*m[14]
            + temp._m[12]*m[15];

    _m[13]= temp._m[1 ]*m[12]
            + temp._m[5 ]*m[13]
            + temp._m[9 ]*m[14]
            + temp._m[13]*m[15];

    _m[14]= temp._m[2 ]*m[12]
            + temp._m[6 ]*m[13]
            + temp._m[10]*m[14]
            + temp._m[14]*m[15];

    _m[15]= temp._m[3 ]*m[12]
            + temp._m[7 ]*m[13]
            + temp._m[11]*m[14]
            + temp._m[15]*m[15];
#endif
}

GLvoid MATRIX::MatrixInverse()
{
    MATRIX temp;

    temp.CopyMatrix(this->_m);

    _m[0 ] = temp._m[0 ];
    _m[1 ] = temp._m[4 ];
    _m[2 ] = temp._m[8 ];

    _m[4 ] = temp._m[1 ];
    _m[5 ] = temp._m[5 ];
    _m[6 ] = temp._m[9 ];

    _m[8 ] = temp._m[2 ];
    _m[9 ] = temp._m[6 ];
    _m[10] = temp._m[10];

    _m[12] *= -1.0f;
    _m[13] *= -1.0f;
    _m[14] *= -1.0f;
}

GLvoid MATRIX::MatrixFromAxisAngle(Vec3 axis, GLfloat theta)
{
    QUAT q;
    GLfloat halfTheta = theta * 0.5f;
    GLfloat cosHalfTheta = fcos(halfTheta);
    GLfloat sinHalfTheta = fsin(halfTheta);
    GLfloat xs, ys, zs, wx, wy, wz, xx, xy, xz, yy, yz, zz;
    q.x = axis.x * sinHalfTheta;
    q.y = axis.y * sinHalfTheta;
    q.z = axis.z * sinHalfTheta;
    q.w = cosHalfTheta;
    xs = q.x * 2;  ys = q.y * 2;  zs = q.z * 2;
    wx = q.w * xs; wy = q.w * ys; wz = q.w * zs;
    xx = q.x * xs; xy = q.x * ys; xz = q.x * zs;
    yy = q.y * ys; yz = q.y * zs; zz = q.z * zs;
    _m[0] = 1 - (yy + zz);
    _m[1] = xy - wz;
    _m[2] = xz + wy;
    _m[4] = xy + wz;
    _m[5] = 1 - (xx + zz);
    _m[6] = yz - wx;
    _m[8] = xz - wy;
    _m[9] = yz + wx;
    _m[10] = 1 - (xx + yy);
    _m[12] = _m[13] = _m[14] = _m[3] = _m[7] = _m[11] = 0;
    _m[15] = 1;
}

GLvoid MATRIX::QuatToMatrix(QUAT quat)
{

    // First row
    _11 = 1.0f - 2.0f * ( quat.y * quat.y + quat.z * quat.z );
    _21 = 2.0f * ( quat.x * quat.y - quat.w * quat.z );
    _31 = 2.0f * ( quat.x * quat.z + quat.w * quat.y );
    _41 = 0.0f;

    // Second row
    _12 = 2.0f * ( quat.x * quat.y + quat.w * quat.z );
    _22= 1.0f - 2.0f * ( quat.x * quat.x + quat.z * quat.z );
    _32 = 2.0f * ( quat.y * quat.z - quat.w * quat.x );
    _42 = 0.0f;

    // Third row
    _13 = 2.0f * ( quat.x * quat.z - quat.w * quat.y );
    _23= 2.0f * ( quat.y * quat.z + quat.w * quat.x );
    _33 = 1.0f - 2.0f * ( quat.x * quat.x + quat.y * quat.y );
    _43 = 0.0f;

    // Fourth row
    _14 = 0;
    _24 = 0;
    _34 = 0;
    _44 = 1.0f;
}


