// OBJECT Class
// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// based on code by  by Alan Baylis 2001
// under GPL or new BSD license
// v 0 (21.10.02): initial release
// v 0.1 (21.10.02): animation methods added
#ifndef ObjectH
#define ObjectH

#include "matrix.h"
#include "Vec3.h"
#include "quat.h"
#include <GL/gl.h>

#include "Context.hpp"


class OBJECT
{
public:
    OBJECT();
    ~OBJECT();

    GLvoid Update();
    GLvoid Apply();

    GLvoid Reset();
    GLvoid Rotate();
    GLvoid Rotate(GLfloat roll, GLfloat pitch, GLfloat yaw);
    GLvoid  Draw();

    GLvoid SetBoundingBox();
    GLvoid SetBoundingSphere();
    GLvoid SetPosition(GLfloat x, GLfloat y, GLfloat z);
    GLvoid GetPosition(GLfloat *x, GLfloat *y, GLfloat *z);
    GLvoid UpdatePosition();
    GLvoid UpdatePosition(GLfloat x, GLfloat y, GLfloat z);
    GLvoid MoveX();
    GLvoid MoveY();
    GLvoid MoveZ();
    GLvoid MoveX(GLfloat x);
    GLvoid MoveY(GLfloat y);
    GLvoid MoveZ(GLfloat z);
    Vec3 GetXUnit();
    Vec3 GetYUnit();
    Vec3 GetZUnit();


    //Animation helper methods
    GLvoid	NextAnim();
    GLvoid	PrevAnim();
    GLvoid	SetCurrentAnim(GLint i);
    GLvoid	SetAnimEnd( GLint i);
    GLvoid	SetFPS(const GLint fps);
    GLint	GetFPS();
    GLint	GetFrameCount();
    GLvoid	SetAnimStart( GLint i);
    GLint	GetCurrentAnim();
    GLint	GetNextAnim();

    Vec3 Position;
    GLfloat Delta_x;   //Rotation deltas
    GLfloat Delta_y;
    GLfloat Delta_z;
    GLfloat Movement_x;    //Movement displacements
    GLfloat Movement_y;
    GLfloat Movement_z;
    GLfloat Multiplier; //FPS value i.e. 1.0f means 60FPS(full speed)

private:
    // a 4x4 matrix to be used to convert a quaternion to a matrix
    MATRIX Matrix;

    QUAT Orientation;


    GLint								_current_animation;
    GLint								_current_animation_frame2;
    GLint								_loop_begin;
    GLint								_loop_end;
    GLint								_fps;
    GLint								_framecount;

};

#endif
