// v 0 (21.10.02): initial release
// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// under GPL or new BSD license
// vec3.h
#include <dc/fmath.h>


#define pi F_PI
#define F_PI2 (2.0f*F_PI)
#define radian  (F_PI / 180.0f)

#define DEG2RAD(a)	(F_PI/180.0f*(a))
#define RAD2DEG(a)	(180.0f/F_PI*(a))

#ifndef VEC3_H
#define VEC3_H

#define CVector3 Vec3

// Based on vector class by George Foot in the playpen demo.

class Vec3
{
public:
    float	x, y, z;

    Vec3 & set( const float _x, const float _y, const float _z)
    { x = _x; y = _y; z = _z; return *this; }

    Vec3 & set( const Vec3 &v )
    { return set(v.x, v.y, v.z); }

    Vec3 & operator=( const Vec3 &v )
    { return set(v); }

    Vec3 (void )
    {}

    Vec3 ( const float _x, const float _y, const float _z)
    { set(_x, _y, _z); }

    Vec3 ( const Vec3 &v )
    { set(v); }

    /* Infix operators */
    Vec3 operator+ (const Vec3 &v)  const { return Vec3 (x+v.x, y+v.y, z+v.z); }
    Vec3 operator- (const Vec3 &v)  const { return Vec3 (x-v.x, y-v.y, z-v.z); }
    Vec3 operator- (void)		    const { return Vec3 (-x, -y, -z); }
    Vec3 operator* (const float f) const { return Vec3 (x*f, y*f, z*f); }
    Vec3 operator/ (const float f) const { return Vec3 (x/f, y/f, z/f); }

    /* In place operators */
    Vec3 &operator+= (const Vec3 &v) { x += v.x; y += v.y; z += v.z; return *this; }
    Vec3 &operator-= (const Vec3 &v) { x -= v.x; y -= v.y; z -= v.z; return *this; }
    Vec3 &operator*= (const float f) { x *= f; y *= f; z *= f; return *this; }
    Vec3 &operator/= (const float f) { x /= f; y /= f; z /= f; return *this; }

    /* Data */

    //FIXME
    float length (void) const { return (float) fsqrt(x*x + y*y + z*z); }
    float lengthsq (void) const { return x*x + y*y + z*z; }

    Vec3 &normalize (void) { return *this /= length(); }
    Vec3 &Normalize (void) { return *this /= length(); }

    // dot product
    float  operator| (const Vec3 &v) const { return x*v.x + y*v.y + z*v.z; }

    // cross product
    Vec3   operator^ (const Vec3 &v) const { return Vec3 (y*v.z - z*v.y, z*v.x - x*v.z, x*v.y - y*v.x); }

    Vec3 rotateY( const float radians )
    {
        float x2 =  fcos(radians) * x + fsin(radians) * z;
        float z2 = -fsin(radians) * x + fcos(radians) * z;
        x = (float)x2; z = (float)z2; return *this; }

    void Reset() {  x = 0;    y = 0;    z = 0;}

    Vec3 GetNormalized()
    {
        return Vec3(*this).normalize();
    }

    static const Vec3 MakeAverage(const Vec3 &a, const Vec3 &b){

    return Vec3(
			(a.x + b.x)*0.5f,
			(a.y + b.y)*0.5f,
			(a.z + b.z)*0.5f
		);
	}

};

#endif
