
// (c) by Heinrich Tillack 2002 (http://a128.x15.org)
// based on code by Jimbo
// under GPL or new BSD license
// v 0 (21.10.02): initial release
//v 0.1 (?): texture.c: search for text maps already loaded into the PVR ram!
//						and loads PNG,JPG and PCX depending of the filename

#include <GL/gl.h>
#include <kos.h>

#include "texture.h"
#include "utils.h"
#include <png/png.h>

#define TRUE 1
#define FALSE 0

#define MAX_TEXTURES 256

GLint texture_ram[MAX_TEXTURES];
struct texture textures[MAX_TEXTURES];

int max_pos=-1;

void texture_init() {
    int i;
    glGenTextures(MAX_TEXTURES,&texture_ram[0]);
    for(i=0;i<MAX_TEXTURES;i++){

        textures[i].txr=NULL;
    }
}

GLint next_tex() {
    static int pos=0;
    GLint ret=texture_ram[pos];
    pos++;
    return ret;
}

// returns the next tex_position or -1 if error!
int next_tex_id() {
    int index=0;

    //find any free text Positions
    while(textures[index].txr!=NULL)
    {
        index++;
        if(index>MAX_TEXTURES) {assert(0);return -1;}
    }

    return index;


}

struct texture *get_tex(int index) {
    return &textures[index];
}

int find_tex(char *filename) {
    int index=0;
    for(index=0;index<=max_pos;index++) {

        if (textures[index].txr!=NULL)
            if(!strcmp(textures[index].filename,filename)) {

                //	DBG("found texture %s\n",textures[index].filename);

                return index;
            }
    }
    return -1;
}

static int old_pos=-1;

void switch_tex(int pos) {

   //bind texture if not already binded :-)
	if(old_pos!=pos){
    glBindTexture(GL_TEXTURE_2D, textures[pos].txr);
    old_pos=pos;
    }
}


void delete_tex(int pos) {

	if(pos<0) return;
	if(pos>MAX_TEXTURES) return;

	 if(textures[pos].txr!=NULL){
    glDeleteTextures(1,&textures[pos].txr);
    textures[pos].txr=NULL;
	}

}


//jpeg loader
void load_jpg_texture(char *filename,int pos,int type) {


    int size;

    textures[pos].txr=next_tex();

    if(pos>max_pos) { max_pos=pos; }   strcpy(textures[pos].filename,filename);

    //     if(type==PCX) pcx_to_txr(filename,&textures[pos].txraddr);
    //     else if(type==JPG)
    jpeg_to_txr(filename, &textures[pos].txraddr,0,1,1,&size);
    //  else assert(0);


    textures[pos].xsize=size; textures[pos].ysize=size;

    glBindTexture(GL_TEXTURE_2D, textures[pos].txr);

    glKosTex2D(GL_RGB565_TWID, textures[pos].xsize,
               textures[pos].ysize,textures[pos].txraddr);


}


//jpeg/png/pcx  loader
//returns the textureID which you can use for switchtex()
int auto_load_texture(char *filename) {


#define file filename

int pos=find_tex(filename);

    //texture map already loaded! v0.2

   if(pos>0) {printf("Material already loaded\n");
    return pos;
   }

   int size;
	int type=-1;

     pos=next_tex_id();

    textures[pos].txr=next_tex();

    if(pos>max_pos) { max_pos=pos; }   strcpy(textures[pos].filename,filename);

     if(strstr(file,".PNG")!=NULL||strstr(file,".png")!=NULL)
     type=PNG;
     else

     if(strstr(file,".JPG")!=NULL||strstr(file,".jpg")!=NULL)
     type=JPG;
     else

     if(strstr(file,".PCX")!=NULL||strstr(file,".pcx")!=NULL)
     type=PCX;


   if(type==PCX) pcx_to_txr(filename,&textures[pos].txraddr,&size);
       else if(type==JPG)
    jpeg_to_txr(filename, &textures[pos].txraddr,0,1,1,&size);
    else if (type==PNG){
	     kos_img_t img;
	pvr_ptr_t txaddr;

	if (png_to_img(filename, PNG_NO_ALPHA, &img) < 0)
	{
		printf("can't load %s\n", filename);
		return 0;
	}

	textures[pos].txraddr =txaddr = pvr_mem_malloc(img.w * img.h * 2);

	pvr_txr_load_kimg(&img, txaddr, 0);


    if(img.w>img.h) size=img.w;
    else size=img.h;

	textures[pos].xsize=size; textures[pos].ysize=size;

	kos_img_free(&img, 0);



	    }

    else
    assert(0);

    textures[pos].xsize=size; textures[pos].ysize=size;

    glBindTexture(GL_TEXTURE_2D, textures[pos].txr);

    glKosTex2D(GL_RGB565_TWID, textures[pos].xsize,
               textures[pos].ysize,textures[pos].txraddr);

    return pos;
}

#undef file
